Index: modules/authentication/src/org/labkey/authentication/ldap/LdapAuthenticationManager.java
===================================================================
--- modules/authentication/src/org/labkey/authentication/ldap/LdapAuthenticationManager.java	(revision 19090)
+++ modules/authentication/src/org/labkey/authentication/ldap/LdapAuthenticationManager.java	(revision )
@@ -26,9 +26,13 @@
 
 import javax.naming.AuthenticationException;
 import javax.naming.Context;
+import javax.naming.InitialContext;
+import javax.naming.NamingEnumeration;
 import javax.naming.NamingException;
 import javax.naming.directory.DirContext;
 import javax.naming.directory.InitialDirContext;
+import javax.naming.directory.SearchControls;
+import javax.naming.directory.SearchResult;
 import java.util.Hashtable;
 import java.util.Map;
 
@@ -64,31 +68,98 @@
     }
 
 
-    // Careful... blank principal or password will switch to "anonymous bind", meaning it will always succeed
+    // Base connect that sends default value to other connect method for the trySearch parameter
     public static boolean connect(String url, @NotNull String principal, @NotNull String password, boolean saslAuthentication) throws NamingException
     {
-        boolean authenticated = false;
+        return connect(url, principal, password, saslAuthentication, true);
+    }
 
-        try
+    // Connection method is used in multiple places
+    private static DirContext connectToLdap(String url, String principal, String password, boolean saslAuthentication) throws NamingException
-        {
+    {
             Hashtable<String, String> env = new Hashtable<String, String>();
             env.put(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory");
             env.put(Context.PROVIDER_URL, url);
+            env.put(Context.REFERRAL, "follow");
             env.put(Context.SECURITY_AUTHENTICATION, saslAuthentication ? "DIGEST-MD5 CRAM-MD5 GSSAPI" : "simple");
             env.put(Context.SECURITY_PRINCIPAL, principal);
             env.put(Context.SECURITY_CREDENTIALS, password);
-            DirContext ctx = new InitialDirContext(env);
+            return new InitialDirContext(env);
+    }
+
+    // Careful... blank principal or password will switch to "anonymous bind",
+    // meaning it will always succeed
+    public static boolean connect(String url, @NotNull String principal,
+                                  @NotNull String password, boolean saslAuthentication,
+                                  boolean trySearch) throws NamingException
+    {
+        boolean authenticated = false;
+        DirContext ctx = null;
+        try
+        {
+            ctx = connectToLdap(url, principal, password, saslAuthentication);
             authenticated = true;
-            ctx.close();
         }
         catch (AuthenticationException e)
         {
+            e.printStackTrace();
+            if (trySearch)
+            {
+                // Here we do the search for sAMAccountName
+                try
+                {
+                    authenticated = trySearchAndAuthenticate(url, principal, password, saslAuthentication);
+                }
+                catch (NamingException ex)
+                {
+                    _log.debug("LDAPAuthenticate: Invalid search account login. JNDI-set credentials incorrect. server="
+                            + url + ", security principal=" + principal);
+                    throw ex;
+                }
+            }
+        }
+        finally
+        {
+            if (!authenticated)
-            _log.debug("LDAPAuthenticate: Invalid login. server=" + url + ", security principal=" + principal);
+                _log.debug("LDAPAuthenticate: Invalid login. server=" + url + ", security principal=" + principal);
+            if (ctx != null)
+                ctx.close();
         }
 
         return authenticated;
     }
 
+    // Login with a default account and then search for the sAMAccountName. If
+    // we find it, redo connection attempt with the new info
+    private static boolean trySearchAndAuthenticate(String url,
+                                                    @NotNull String principal, @NotNull String password,
+                                                    boolean saslAuthentication) throws NamingException
+    {
+        Context jndi = (Context) new InitialContext().lookup("java:comp/env");
+        DirContext ctx = connectToLdap(url,
+                (String) jndi.lookup("ldap_username"),
+                (String) jndi.lookup("ldap_password"), saslAuthentication);
+        try
+        {
+            SearchControls searchCtls = new SearchControls();
+            // searchCtls.setReturningAttributes(returnedAtts);
+            // Specify the search scope
+            String searchFilter = "(&(objectClass=user)(mail=" + principal + "))";
+
+            searchCtls.setSearchScope(SearchControls.SUBTREE_SCOPE);
+            String defaultSearchBase = "DC=urmc-sh,DC=rochester,DC=edu";
+            NamingEnumeration<SearchResult> results = ctx.search(defaultSearchBase, searchFilter, searchCtls);
+            return connect(url,
+                    results.next().getAttributes().get("sAMAccountName").get()
+                            .toString(), password, saslAuthentication, false); // turn off trySearch for this attempt
+        }
+        finally
+        {
+            if (ctx != null)
+                ctx.close();
+        }
+    }
+
     private static final String LDAP_AUTHENTICATION_CATEGORY_KEY = "LDAPAuthentication";
 
     public static void activate()
